/* Copyright (C) Arm Limited, 2020 All rights reserved. 
 * The example code is provided to you as an aid to learning when working 
 * with Arm-based technology, including but not limited to programming tutorials. 
 * Arm hereby grants to you, subject to the terms and conditions of this Licence, 
 * a non-exclusive, non-transferable, non-sub-licensable, free-of-charge licence, 
 * to use and copy the Software solely for the purpose of demonstration and 
 * evaluation.
 * You accept that the Software has not been tested by Arm therefore the Software 
 * is provided "as is", without warranty of any kind, express or implied. In no 
 * event shall the authors or copyright holders be liable for any claim, damages 
 * or other liability, whether in action or contract, tort or otherwise, arising 
 * from, out of or in connection with the Software or the use of Software.
 */

#include <stdio.h>
#include <arm_mve.h> 

void my_mult_f32_intr(
                float32_t * pSrcA, float32_t * pSrcB,
                float32_t * pDst, uint32_t blockSize) {

  // Calculate memory block size for 4 x lanes of float32_t data
  const int blkSize_F32 = 4 * sizeof(float32_t);

  // Calculate how many loop iterations are required:
  //    size of array / size of 4 data items
  int blkCnt = blockSize / blkSize_F32;

  // Create source and destination vectors, configured for 4 lanes of float32_t data
  float32x4_t vecA, vecB, vecDst;


  // Main loop
  while (blkCnt > 0U) {
    // Load source vectors with data from the input arrays
    vecA = vldrwq_f32(pSrcA);
    vecB = vldrwq_f32(pSrcB);

    // Perform vector multiplication
    vecDst = vmulq_f32(vecA, vecB);

    // Store the result vector into the destination array
    vstrwq_f32(pDst,  vecDst);

    // Decrement the loop count
    blkCnt--;

    // Advance source and destination pointer addresses by the size of 4 data elements
    pSrcA += blkSize_F32;
    pSrcB += blkSize_F32;
    pDst += blkSize_F32;
  }
}

int main() {
  // Setup data in input arrays
  float32_t A_src[] = {1.1, 7.9, 8.2, 2.1, 5.3, 2.2, 3.1, 6.9};
  float32_t B_src[] = {7.2, 2.7, 9.9, 8.2, 1.3, 1.1, 6.9, 2.4};

  // Call the multiplication function
  my_mult_f32_intr(&A_src[0], &B_src[0], &A_src[0], sizeof(A_src));

  return 0;
}

