/* Copyright (C) Arm Limited, 2020 All rights reserved. 
 * The example code is provided to you as an aid to learning when working 
 * with Arm-based technology, including but not limited to programming tutorials. 
 * Arm hereby grants to you, subject to the terms and conditions of this Licence, 
 * a non-exclusive, non-transferable, non-sub-licensable, free-of-charge licence, 
 * to use and copy the Software solely for the purpose of demonstration and 
 * evaluation.
 * You accept that the Software has not been tested by Arm therefore the Software 
 * is provided "as is", without warranty of any kind, express or implied. In no 
 * event shall the authors or copyright holders be liable for any claim, damages 
 * or other liability, whether in action or contract, tort or otherwise, arising 
 * from, out of or in connection with the Software or the use of Software.
 */

#include <arm_mve.h>
#include <arm_math.h>
#include <stdio.h>

void my_cmplx_dot_prod_q31(q31_t * pSrcA, q31_t * pSrcB, uint32_t numSamples, q63_t * realResult, q63_t * imagResult) {
  __asm volatile (
        "   clrm                    {r4-r7}                 \n"
        "   wlstp.32                lr, %[cnt], 1f          \n"
        "2:                                                 \n"
        "   vldrw.32                q0, [%[pA]], 16         \n"
        "   vldrw.32                q1, [%[pB]], 16         \n"
        "   vrmlsldavha.s32         r4, r5, q0, q1          \n"
        "   vrmlaldavhax.s32        r6, r7, q0, q1          \n"
        "   letp                    lr, 2b                  \n"
        "1:                                                 \n"
        "   asrl                    r4, r5, #6              \n"
        "   asrl                    r6, r7, #6              \n"
        "   strd                    r4, r5, [%[realResult]] \n"
        "   strd                    r6, r7, [%[imagResult]] \n"
        :[pA] "+r"(pSrcA),[pB] "+r"(pSrcB)
        :[cnt] "r"(numSamples * 2), [realResult] "r"(realResult), [imagResult] "r"(imagResult)
        :"r4", "r5", "r6", "r7", "lr", "memory");
}



int main() {
  // Setup data in input arrays
  q31_t A_src[] = {947483647, 834662098, 111222333, 555666777, 101202303, 555000222, 432654876, 999888777};
  q31_t B_src[] = {147483647, 623333999, 623957233, 876543098, 337744884, 112233445, 909808707, 543098765};

  // Create pointers to the two input arrays
  q31_t *pA_src = A_src;
  q31_t *pB_src = B_src;

  // Setup result variables
  q63_t res_real;
  q63_t res_imag;

  // Create pointers to the two result variables
  q63_t *pres_real = &res_real;
  q63_t *pres_imag = &res_imag;


  // Get the number of elements in the array
  int num_array_elements = sizeof(A_src) / sizeof(q31_t);

  // Divide by 2 to get the number of vector elements 
  //  (each vector element is a pair: a real and a complex component)
  int num_vector_elements = num_array_elements / 2;

  // Call the dot product function, reusing one of the input arrays as the result array
  my_cmplx_dot_prod_q31(pA_src, pB_src, num_vector_elements, pres_real, pres_imag);

  // Print the result
  printf("\n\nreal=%lld ; complex=%lld\n", (long long)res_real, (long long)res_imag);

 return 0;
}

